@@ -6,6 +6,9 @@
 int main(int argc, char *argv[]) {
     QApplication app(argc, argv);
 
+    app.setOrganizationName("Crochik");
+    app.setApplicationName("GeePS");
+
     MainWindow wnd;
     // MapWindow wnd;
     // app.m_pMainWindow = &wnd;
