@@ -16,6 +16,9 @@
 
 #include <QStringBuilder>
 
+#include <QSettings>
+#include <QShortcut>
+
 #include <pickselectoraction.h>
 
 #ifdef Q_WS_MAEMO_5
@@ -44,6 +47,9 @@ MainWindow::MainWindow(QWidget *parent) :
 
     m_pGoogleDirections = NULL;
     m_pAutoRotateAction = NULL;
+    m_pTrafficLayer = NULL;
+    m_pCyclingLayer = NULL;
+
     m_isAutoCenter = true;
     m_isAutoRotate = false;
     m_angle =0;
@@ -150,7 +156,7 @@ void MainWindow::init() {
     grabZoomKeys(true);
 #endif
 
-    ui->webView->setUrl( QUrl::fromUserInput("http://www.crochik.com/gps.html") );
+    ui->webView->setUrl( QUrl::fromUserInput("http://www.crochik.com/gps1.html") );
 
     MouseSuppressor *pSupperssor = new MouseSuppressor(ui->webView);
     connect(pSupperssor, SIGNAL(click(QMouseEvent*)), SLOT(onClick(QMouseEvent*)));
@@ -173,11 +179,15 @@ void MainWindow::createMenu() {
     connect(m_pViewGroup, SIGNAL(triggered(QAction*)), this, SLOT(onViewChanged(QAction*)));
     m_pViewGroup->setExclusive(true);
 
+    QSettings settings;
+
+    int viewMode = settings.value("Mode", 1).toInt();
+
     QAction *pAction;
-    pAction = new QAction("Hybrid", m_pViewGroup); pAction->setCheckable(true);
-    pAction = new QAction("Road Map", m_pViewGroup); pAction ->setCheckable(true); pAction->setChecked(true);
-    pAction = new QAction("Satellite", m_pViewGroup); pAction->setCheckable(true);
-    pAction = new QAction("Terrain", m_pViewGroup); pAction->setCheckable(true);
+    pAction = new QAction("Hybrid", m_pViewGroup); pAction->setCheckable(true); pAction->setChecked(viewMode==0);
+    pAction = new QAction("Road Map", m_pViewGroup); pAction ->setCheckable(true); pAction->setChecked(viewMode==1);
+    pAction = new QAction("Satellite", m_pViewGroup); pAction->setCheckable(true); pAction->setChecked(viewMode==2);
+    pAction = new QAction("Terrain", m_pViewGroup); pAction->setCheckable(true); pAction->setChecked(viewMode==3);
     menuBar()->addActions(m_pViewGroup->actions());
 
     // menuBar()->addAction("Reload", this, SLOT(reload()));
@@ -193,9 +203,26 @@ void MainWindow::createMenu() {
     m_pAutoRotateAction->setChecked(false);
     */
 
+    bool traffic = settings.value("TrafficLayer", false).toBool();
+    bool bicycle = settings.value("BicyclingLayer", false).toBool();
+    m_pTrafficLayer = menuBar()->addAction("Traffic Layer", this, SLOT(onTrafficLayer()));
+    m_pTrafficLayer->setCheckable(true);
+    m_pTrafficLayer->setChecked(traffic);
+
+    m_pCyclingLayer = menuBar()->addAction("Bicycling Layer", this, SLOT(onBicyclingLayer()));
+    m_pCyclingLayer->setCheckable(true);
+    m_pCyclingLayer->setChecked(bicycle);
+
+    if ( traffic ) onTrafficLayer();
+    if ( bicycle ) onBicyclingLayer();
+
+
 #ifdef Q_WS_MAEMO_5
     addFullscreenToMenu();
     addOrientationToMenu();
+    // new QShortcut(QKeySequence("F"), this, SLOT(onFullscreen()));
+
+    int updates = settings.value("UpdateMode", 0).toInt();
 
     m_pSourceUpdatesBtt = new PickSelectorAction(this);
     m_pSourceUpdatesBtt->setText("Position Updates");
@@ -205,12 +232,38 @@ void MainWindow::createMenu() {
             // << "10 per second" << "5 per second"
             // << "every minute" << "every 5 minutes" << "every 10 minutes" << "every 15 minutes" << "every half hour" << "every hour";
 
+    m_pSourceUpdatesBtt->setSelectedIndex(updates);
+
     connect(m_pSourceUpdatesBtt, SIGNAL(selectionChanged(int)), this, SLOT(onSourceUpdates(int)));
     menuBar()->addAction(m_pSourceUpdatesBtt);
-
+    onSourceUpdates(updates);
 #endif
 }
 
+void MainWindow::onTrafficLayer() {
+    QSettings settings;
+    settings.setValue("TrafficLayer", m_pTrafficLayer->isChecked());
+
+    if ( m_pTrafficLayer->isChecked() ) {
+        runJavascript("trafficLayer = new google.maps.TrafficLayer(); trafficLayer.setMap(map);");
+        return;
+    }
+
+    runJavascript("if (trafficLayer) { trafficLayer.setMap(null); trafficLayer=null; }");
+}
+
+void MainWindow::onBicyclingLayer() {
+    QSettings settings;
+    settings.setValue("BicyclingLayer", m_pCyclingLayer->isChecked());
+
+    if ( m_pCyclingLayer->isChecked() ) {
+        runJavascript("bikeLayer = new google.maps.BicyclingLayer(); bikeLayer.setMap(map);");
+        return;
+    }
+
+    runJavascript("if (bikeLayer) { bikeLayer.setMap(null); bikeLayer=null; }");
+}
+
 void MainWindow::onAutoRotate() {
     m_isAutoRotate = m_pAutoRotateAction->isChecked();
     if ( !m_isAutoRotate ) {
@@ -248,6 +301,9 @@ void MainWindow::onSourceUpdates(int index) {
         case 12: updates = 1; break;
     }
 
+    QSettings settings;
+    settings.setValue("UpdateMode", index);
+
 #ifdef Q_WS_MAEMO_5
     m_geoLocation.setUpdatesPerHour(updates);
 #endif
@@ -281,8 +337,9 @@ void MainWindow::onHideButtons() {
 void MainWindow::onViewChanged(QAction* pAction) {
     // google.maps.MapTypeId. // HYBRID ROADMAP SATELLITE TERRAIN
 
+    int index = m_pViewGroup->actions().indexOf(pAction);
     QString value;
-    switch ( m_pViewGroup->actions().indexOf(pAction) ) {
+    switch ( index ) {
         case 0: value = "HYBRID"; break;
         case 1: value = "ROADMAP"; break;
         case 2: value = "SATELLITE"; break;
@@ -291,6 +348,9 @@ void MainWindow::onViewChanged(QAction* pAction) {
             return;
     }
 
+    QSettings settings;
+    settings.setValue("Mode", index);
+
     runJavascript("map.setMapTypeId(google.maps.MapTypeId." + value + ");");
 }
 
@@ -647,6 +707,23 @@ void MainWindow::keyPressEvent(QKeyEvent *event) {
             // str = "updateCenter(true);";
             return;
 
+        case Qt::Key_F:
+            onFullscreen();
+            return;
+
+        case Qt::Key_0: // = 0x30,
+        case Qt::Key_1: // = 0x31,
+        case Qt::Key_2: // = 0x32,
+        case Qt::Key_3: // = 0x33,
+        case Qt::Key_4: // = 0x34,
+        case Qt::Key_5: // = 0x35,
+        case Qt::Key_6: // = 0x36,
+        case Qt::Key_7: // = 0x37,
+        case Qt::Key_8: // = 0x38,
+        case Qt::Key_9: // = 0x39,
+            str = QString("map.setZoom(%1);").arg((event->key()-Qt::Key_0)*2);
+            break;
+
         default:
             return;
     }
